#include "header.h"

void put_manual () {
  GtkWidget *window, *grid;
  GtkWidget *scrolled, *sgrid, *label1, *label2;
  GtkWidget *button;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "NeonView Manual");
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_widget_set_size_request(window, 520, 480);
  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);

  grid = gtk_grid_new();
  gtk_container_add(GTK_CONTAINER(window), grid);

  char *str1="<b>NeonView Manual</b>\n\n<b>Introduction</b>\n\nNeonView is a fast, lightweight image viewer written in C and using the GTK+ 3 toolkit. It has support for about 30 image formats, including the most popular ones like JPG, PNG, SVG, GIF, TGA and TIFF. Currently it is available on Linux, but a port for Windows is planned too.\n\nNeonView strives for simplicity, but still provides some handy features like thumbnail previews, basic image manipulation functions like rotating images, zooming, as well as configuration options to customize the interface and the way it behaves. NeonView will keep configuration changes between sessions, it will save window size and it will remember if images should be displayed at full size rather than fit to window. All the configuration options can be changed from within NeonView or from the <b>neonview.cfg</b> config file.\n\n<b>Features</b>\n\nSee the features.html file which came in the NeonView package.\n\n<b>List of supported formats</b>\n\nNeonView doesn't do any image parsing or format recognition by itself, instead it uses GdkPixbuf. Any format supported by GdkPixbuf is supported by NeonView. The following formats are currently supported:\n\nPNG\tICO\tCUR\tANI\tRAS\nPCX\tXBM\tTGA\tSVG\tJPG\nTIFF\tXPM\tBMP\tPNM\tPBM\nPGM\tPPM\tQIF\tJPC\tJPX\nJ2K\tJPF\tJPX\tGIF\tICNS\n\n<b>Configuration</b>\n\nMost of the configuration options can be changed from within NeonView itself, however there are several options which can only be changed in the configuration file. The configuration file is <b>~/.neonview/neonview.cfg</b>, where <b>~</b> is your home directory (e.g. /home/username/.neonview/neonview.cfg). This file is a text file and can be edited with any text editor. Keep in mind that <b>.neonview</b> is a hidden directory. NeonView saves its options into this file when it exits and loads all the options here when it starts up. If the file is corrupted, NeonView will proceed with default settings. You are encouraged to edit this file since any bad editing will not break NeonView. NeonView will echo to the standard output (run it in the console to see the output) any error it may find in this file. Everything after the <b>#</b> sign is a comment and will be ignored by NeonView. Every line should have the form <b>KEY=\"VALUE\"</b>, however double-quotes are not mandatory, and spaces can be used since NeonView will strip those when interpreting the file. Here are the available options and values which can be edited in this file:\n\n<b>last_dir</b> # can be any string\nThis is the last directory known by NeonView before it exited.\n<b>start_dir</b> # can be any string\nThis is the start-up directory, NeonView will start in this directory if no other argument is passed to it\n\n<b>window_width</b> # integer between 0 and 2440 * 2\nThe main window width\n\n<b>window_height</b> # integer between 0 and 1440 * 2\nThe main window height\n\n<b>start_maximized</b> # can be \"yes\" or\"no\"\nTells NeonView whether to start maximized or not\n\n<b>show_toolbar</b> # can be \"yes\" or \"no\"\nShow toolbar\n\n<b>show_status</b> # can be \"yes\" or \"no\"\nShow the status bar\n\n<b>show_left_sidebar</b> # can be \"yes\" or \"no\"\nShow the thumbnail previews panel\n\n<b>show_right_sidebar</b> # can be \"yes\" or \"no\"\nShow the info panel\n\n<b>remember_fit</b> # can be \"yes\" or \"no\"\nRemeber between sessions if NeonView should automatically fit images to window or not (only if the <b>Fit to Window</b> button was active when NeonView exited)\n\n<b>remember_zoom</b> # can be \"yes\" or \"no\"\nRemember the zoom level between sessions\n\n<b>fit_next</b> # can be \"yes\" or \"no\"\nIf this option is enabled, once the <b>Fit to Window</b> button has been pressed, NeonView will display all the images available via Previous and Next buttons resized to fit to window\n\n";

char *str2 = "<b>Feature requests, bugs, suggestions</b>\n\nFor feature requests, bug reports or suggestions you have several options:\n\nSubmit them on the website via post comments at:\n\n\t<a href=\"http://www.tuxarena.com/neonview/\">http://www.tuxarena.com/neonview/</a>\n\nUse the forum at:\n\n\t<a href=\"http://www.tuxarena.com/forum/\">http://www.tuxarena.com/forum/</a>\n\nContact me at:\n\n\t<a href=\"mailto:craciun.dan@tuxarena.com\">craciun.dan@tuxarena.com</a>";

  scrolled = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_vexpand(scrolled, TRUE);
  gtk_grid_attach(GTK_GRID(grid), scrolled, 0, 0, 1, 1);

  sgrid = gtk_grid_new();
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), sgrid);

  char *markup;

  markup = g_markup_printf_escaped(str1);
  label1 = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label1), markup);
  //  gtk_label_set_max_width_chars(GTK_LABEL(label1), 40);
  gtk_label_set_line_wrap(GTK_LABEL(label1), TRUE);
  gtk_label_set_line_wrap_mode(GTK_LABEL(label1), PANGO_WRAP_WORD_CHAR);
  gtk_grid_attach(GTK_GRID(sgrid), label1, 0, 0, 1, 1);

  markup = g_markup_printf_escaped(str2);
  label2 = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label2), markup);
  //  gtk_label_set_max_width_chars(GTK_LABEL(label2), 40);
  gtk_label_set_line_wrap(GTK_LABEL(label2), TRUE);
  gtk_label_set_line_wrap_mode(GTK_LABEL(label2), PANGO_WRAP_WORD_CHAR);
  gtk_grid_attach(GTK_GRID(sgrid), label2, 0, 1, 1, 1);

  button = gtk_button_new_with_label("OK");
  gtk_widget_set_hexpand(button, TRUE);
  gtk_widget_set_margin_top(button, 10);
  gtk_grid_attach(GTK_GRID(grid), button, 0, 1, 1, 1);

  g_signal_connect_swapped(button, "clicked", G_CALLBACK(about_ok), window);

  GtkAccelGroup *accel_group;
  accel_group = gtk_accel_group_new();
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
  gtk_widget_add_accelerator(button, "clicked", accel_group, GDK_KEY_Escape, 0, GTK_ACCEL_VISIBLE);

  gtk_widget_show_all(grid);
  gtk_widget_show(window);
}

void put_faq () {
  GtkWidget *window, *grid;
  GtkWidget *scrolled, *label;
  GtkWidget *button;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "NeonView FAQ");
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_widget_set_size_request(window, 520, 480);
  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);

  grid = gtk_grid_new();
  gtk_container_add(GTK_CONTAINER(window), grid);

  char *markup;
  char *str = "<b>NeonView FAQ v0.8.0</b>\n\n<b>Q: What is it with these codenames?</b>\nA: The codenames are inspired by the genus or species names of various fish.\n\n<b>Q: Why does NeonView start maximized when the Start maximized option is  disabled?</b>\nA: NeonView always remembers the size (width and height) of the main window. So if you close NeonView and it is maximized, the next time it starts the window will read the width and height values, even though the Start maximized option is disabled. To prevent it from doing so (and just always start with the default, hard-coded size), uncheck the Remember window size option too in the <b>Tools->Settings</b> window.\n\n<b>Q: How does NeonView handle images?</b>\n\nA: NeonView supports all formats handled by GdkPixbuf, so it doesn't read anything by itself. No code involving format recognition is involved, everything is being taken care of by GdkPixbuf. Since 0.8.0, NeonView correctly displays animated GIF files too.\n\nSupported formats include JPG, PNG, SVG, BMP, GIF, TGA, TIFF, ICO and ANI.\n\n<b>Q: Is there any configuration file available?</b>\nA: Yes. The file is called ~/.neonview/neonview.cfg, where ~ is your home directory (e.g. /home/user). In this file you can tweak several options that cannot be changed from within NeonView.\n\n<b>Q: Can I disable the left and right panels?</b>\nA: Yes. You can do so by unchecking the respective options in the Tools menu, or by setting show_left_sidebar and show_right_sidebar options to no in the config file. The same applies for the toolbar and the status bar.\n\n<b>Q: Is there a Windows port?</b>\nA: Not yet, but chances are a port will be made, probably when NeonView hits version 1.0. No ports for Mac OS are planned yet. No ports for mobile platforms are planned yet either (this may change though).\n\n<b>Q: On what platforms can I run NeonView?</b>\nA: NeonView is written in C and GTK+ 3 and it has only been tested on Linux (Debian 7.0 Wheezy, Ubuntu 12.04, Mint 14). It works fine on these operating systems.\n\n<b>Q: What license does NeonView use?</b>\nA: GPLv2. This means you are free to modify and redistribute NeonView as long as you keep the same license terms for your derivative work. Please give me credit for my work.\n\n<b>Q: Does it work on older Linux distributions?</b>\nA: It should. However, being GTK+ 3, the look will probably be not quite pleasant.";

  scrolled = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_vexpand(scrolled, TRUE);
  gtk_grid_attach(GTK_GRID(grid), scrolled, 0, 0, 1, 1);

  markup = g_markup_printf_escaped(str);
  label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), markup);
  gtk_label_set_max_width_chars(GTK_LABEL(label), 40);
  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
  gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD_CHAR);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled), label);

  button = gtk_button_new_with_label("OK");
  gtk_widget_set_hexpand(button, TRUE);
  gtk_widget_set_margin_top(button, 10);
  gtk_grid_attach(GTK_GRID(grid), button, 0, 1, 1, 1);

  g_signal_connect_swapped(button, "clicked", G_CALLBACK(about_ok), window);

  GtkAccelGroup *accel_group;
  accel_group = gtk_accel_group_new();
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
  gtk_widget_add_accelerator(button, "clicked", accel_group, GDK_KEY_Escape, 0, GTK_ACCEL_VISIBLE);

  gtk_widget_show_all(grid);
  gtk_widget_show(window);
}

void put_homepage () {
  if (fisfile("/usr/bin/firefox")) {
    system("firefox http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/google-chrome")) {
    system("google-chrome http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/opera")) {
    system("opera http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/konqi")) {
    system("konqi http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/epiphany-browser")) {
    system("epiphany-browser http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/konqueror")) {
    system("konqueror http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/dillo")) {
    system("dillo http://www.tuxarena.com/neonview/ &");
    return;
  }
  if (fisfile("/usr/bin/xterm")) {
    if (fisfile("/usr/bin/w3m")) {
      system("xterm -e w3m http://www.tuxarena.com/neonview/ &");
      return;
    }
    if (fisfile("/usr/bin/lynx")) {
      system("xterm -e lynx http://www.tuxarena.com/neonview/ &");
      return;
    }
  }
}

void put_about () {
  GtkWidget *window, *grid;
  GtkWidget *name, *desc, *author, *site;
  GtkWidget *button;

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "About NeonView");
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_widget_set_size_request(window, 340, 180);
  gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(window), 10);

  grid = gtk_grid_new();
  gtk_container_add(GTK_CONTAINER(window), grid);

  char *markup;

  name = gtk_label_new(NULL);
  markup = g_markup_printf_escaped("<b>%s %s '%s'</b>", NAME, VERSION, CODENAME);
  gtk_label_set_markup(GTK_LABEL(name), markup);
  gtk_widget_set_margin_top(name, 10);
  gtk_grid_attach(GTK_GRID(grid), name, 0, 0, 1, 1);

  desc = gtk_label_new(NULL);
  markup = g_markup_printf_escaped("A lightweight image viewer");
  gtk_label_set_markup(GTK_LABEL(desc), markup);
  gtk_widget_set_margin_top(desc, 10);
  gtk_grid_attach(GTK_GRID(grid), desc, 0, 1, 1, 1);

  author = gtk_label_new(NULL);
  markup = g_markup_printf_escaped("Copyright (C) 2011-2013 Craciun Dan");
  gtk_label_set_markup(GTK_LABEL(author), markup);
  gtk_widget_set_margin_top(author, 10);
  gtk_grid_attach(GTK_GRID(grid), author, 0, 2, 1, 1);

  site = gtk_label_new(NULL);
  markup = g_markup_printf_escaped("<a href=\"http://www.tuxarena.com/neonview/\">http://www.tuxarena.com/neonview/</a>");
  gtk_label_set_markup(GTK_LABEL(site), markup);
  gtk_widget_set_margin_top(site, 10);
  gtk_grid_attach(GTK_GRID(grid), site, 0, 3, 1, 1);

  button = gtk_button_new_with_label("OK");
  gtk_widget_set_hexpand(button, TRUE);
  gtk_widget_set_margin_top(button, 10);
  gtk_grid_attach(GTK_GRID(grid), button, 0, 4, 1, 1);
  g_signal_connect_swapped(button, "clicked", G_CALLBACK(about_ok), window);

  gtk_widget_modify_font(GTK_WIDGET(name), pango_font_description_from_string("DejaVu Sans 13"));
  gtk_widget_modify_font(GTK_WIDGET(desc), pango_font_description_from_string("DejaVu Sans 12"));
  gtk_widget_modify_font(GTK_WIDGET(author), pango_font_description_from_string("DejaVu Sans 10"));
  gtk_widget_modify_font(GTK_WIDGET(site), pango_font_description_from_string("DejaVu Sans 12"));

  GtkAccelGroup *accel_group;
  accel_group = gtk_accel_group_new();
  gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);
  gtk_widget_add_accelerator(button, "clicked", accel_group, GDK_KEY_Escape, 0, GTK_ACCEL_VISIBLE);


  gtk_widget_show_all(grid);
  gtk_widget_show(window);
}

void about_ok (GtkWidget *widget) {
  gtk_widget_destroy(widget);
}
