#include "header.h"

void init_args (TM *tm) {
  int go = 0; // tells program which way to continue

  if (tm->argc == 1) {
    go = 1; // no arguments
  }
  if (tm->argc == 2) {
    if (strcmp(tm->argv[1], "-v") == 0 || strcmp(tm->argv[1], "--version") == 0) {
      fprintf(stdout, "%s %s - %s\n", NAME, VERSION, HTTP);
      exit(0);
    }
    if (strcmp(tm->argv[1], "-h") == 0 || strcmp(tm->argv[1], "--help") == 0) {
      fprintf(stdout, "Usage:\n");
      fprintf(stdout, "  neonview [OPTION|FILE]\n");
      fprintf(stdout, "\n");
      fprintf(stdout, "NeonView is a fast, lightweight image viewer.\n");
      fprintf(stdout, "\n");
      fprintf(stdout, "Options:\n");
      fprintf(stdout, "  -v, --version\t\t\tShow version\n");
      fprintf(stdout, "  -h, --help\t\t\tShow help\n");
      fprintf(stdout, "\n");
      fprintf(stdout, "Examples:\n");
      fprintf(stdout, "  neonview guitar.jpg\t\tOpen image guitar.jpg\n");
      fprintf(stdout, "  neonview -h\t\t\tShow this help\n");
      exit(0);
    }
    if (strcmp(tm->argv[1], "-v") != 0 && strcmp(tm->argv[1], "--version") != 0 &&
	strcmp(tm->argv[1], "-h") != 0 && strcmp(tm->argv[1], "--help") != 0) {
      go = 2; // only one argument, NOT a valid option => try to display as image
    }
  }
  if (tm->argc > 2) {
    fprintf(stdout, "Usage: neonview [OPTION|FILE]\n");
    exit(0);
  }

  init_struct(tm);

  get_cwd(tm);
  get_home(tm);
  get_config_dir(tm);
  get_config_file(tm);

  init_keys(tm); // init keys with default values
  read_config(tm);

  put_window(tm);
  //  put_left_sidebar(tm);
  put_menubar(tm);

  if (strcmp(tm->tcfg->remember_fit, "yes") == 0 && strcmp(tm->tcfg->fit_next, "yes") == 0) {
    strcpy(tm->tcfg->fit_next, "yes");
  } else {
    strcpy(tm->tcfg->fit_next, "no");
  }

  if (go == 2) {
    tm->tnfo->image_path = malloc((strlen(tm->argv[1]) + 1) * sizeof(char));
    if (!tm->tnfo->image_path) {
      criterr(0);
    }
    strcpy(tm->tnfo->image_path, tm->argv[1]);
    if (fisfile(tm->argv[1]) == 1) {
      get_full_path(tm);
      get_dirname(tm);
      show_image(tm);
    } else {
      fprintf(stdout, "[NeonView] WARNING: Could not open file %s for reading.\n", tm->argv[1]);
    }
  }

  if (go == 1) { // no arguments provided
    tm->tnfo->image_path = malloc(1);
    if (!tm->tnfo->image_path) {
      criterr(0);
    }
    strcpy(tm->tnfo->image_path, "");
    get_full_path(tm);
    get_dirname(tm);
    fill_image_list(tm);
    if (tm->tnfo->total > 0) {
      GError **error = NULL;

      if (check_ext(tm->tnfo->image_list[0]) == 2) {
	tm->tgtk->animation = gdk_pixbuf_animation_new_from_file(tm->tnfo->image_list[0], error);
	tm->tgtk->image = gtk_image_new_from_animation(tm->tgtk->animation);
	tm->tgtk->animation_orig = tm->tgtk->animation;
      } else {
      if (strcmp(tm->tcfg->fit_next, "yes") == 0) {
	int width, height;
	gtk_window_get_size(GTK_WINDOW(tm->tgtk->window), &width, &height);
	tm->tgtk->pixbuf = gdk_pixbuf_new_from_file_at_scale(tm->tnfo->image_list[0], width - 300, height - 100, TRUE, NULL);
      } else {
	tm->tgtk->pixbuf = gdk_pixbuf_new_from_file(tm->tnfo->image_list[0], error);
      }
	tm->tgtk->image = gtk_image_new_from_pixbuf(tm->tgtk->pixbuf);
	tm->tgtk->pixbuf_orig = gdk_pixbuf_new_from_file(tm->tnfo->image_list[0], NULL);
      }
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(tm->tgtk->scrolled), tm->tgtk->image);
      gtk_widget_show(tm->tgtk->image);
      tm->tnfo->pos = 0;
      fill_right_sidebar(tm);
    }
  }
  put_left_sidebar(tm);
  put_status(tm);
  put_menubar(tm);

  if (strcmp(tm->tcfg->show_left_sidebar, "yes") == 0) {
    gtk_widget_show_all(tm->tgtk->grid13);
  }

  if (tm->tnfo->total > 0) {
    if (tm->tnfo->pos == tm->tnfo->total - 1) {
      // button next becomes inactive
      gtk_widget_set_sensitive(tm->tgtk->menu_next, FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(tm->tgtk->next), FALSE);
    
    }
    if (tm->tnfo->pos == 0) {
      // button prev becomes inactive
      gtk_widget_set_sensitive(tm->tgtk->menu_prev, FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(tm->tgtk->prev), FALSE);
    }
  }
  // there we go...
  gtk_widget_show(tm->tgtk->window);
}

void get_cwd (TM *tm) {
  char *rv;

  rv = getcwd(tm->tenv->cwd, sizeof(tm->tenv->cwd));
  if (!rv) {
    criterr(1);
  }
}

void get_home (TM *tm) {
  struct passwd *pw;

  pw = getpwuid(getuid());
  tm->tenv->home = malloc((strlen(pw->pw_dir) + 1) * sizeof(char));
  if (!tm->tenv->home) {
    criterr(0);
  }
  strcpy(tm->tenv->home, pw->pw_dir);
}

void get_config_dir (TM *tm) {
  // a more elegant solution to this?
  tm->tenv->config_dir = malloc((strlen(tm->tenv->home) + strlen("/.neonview") + 1) * sizeof(char));
  if (!tm->tenv->config_dir) {
    criterr(0);
  }

  strcpy(tm->tenv->config_dir, tm->tenv->home);
  strcat(tm->tenv->config_dir, "/.neonview");
}

void get_config_file (TM *tm) {
  tm->tenv->config_file = malloc((strlen(tm->tenv->config_dir) + strlen("/.neonview.cfg") + 1) * sizeof(char));
  if (!tm->tenv->config_file) {
    criterr(0);
  }

  strcpy(tm->tenv->config_file, tm->tenv->config_dir);
  strcat(tm->tenv->config_file, "/neonview.cfg");
}

void read_config (TM *tm) {
  FILE *file;

  file = fopen(tm->tenv->config_file, "rt");
  if (!file) {
    // config file not found, try to replace with default
    // if can't write default, just load default values and issue a warning
    write_default_config(tm);
    load_default_config(tm);
    return;
  }
  fprintf(stdout, "[NeonView] Config file %s found.\n", tm->tenv->config_file);

  // now read config values
  char c;
  int i = 0, j = 0;
  int iskey = 1, isval = 0;
  char key[257], val[257];
  int skip = 0;

  while ((c = fgetc(file)) != EOF) {
    if (skip == 0) {
      if (c == '#') {
	skip = 1;
      }
      if (c == '\n') {
	key[i] = '\0';
	val[j] = '\0';
	if (strlen(key) > 0 && strlen(val) > 0) {
	  // we have a key val pair
	  remove_spaces(val);
	  assign_keys(tm, key, val);
	}
	if (strlen(key) > 0 && j == 0) {
	  fprintf(stdout, "[NeonView] WARNING: Invalid config line: %s\n", key);
	  fprintf(stdout, "[NeonView] WARNING: Ignored.\n");
	}
	i = 0;
	j = 0;
	iskey = 1;
	isval = 0;
      }
      if (c != '#' && c != '\n') {
	if (c != '=') {
	  if (iskey == 1) {
	    key[i] = c;
	    i++;
	  }
	  if (isval == 1) {
	    val[j] = c;
	    j++;
	  }
	} else {
	  iskey = 0;
	  isval = 1;
	}
      }
    } else { // skip != 0
      if (c == '\n') {
	skip = 0;
	key[i] = '\0';
	val[j] = '\0';
	i = 0;
	j = 0;
	iskey = 1;
	isval = 0;
	if (strlen(key) > 0 && strlen(val) > 0) {
	  // we have a key val pair
	  remove_spaces(val);
	  assign_keys(tm, key, val);
	}
      }
    }
  }
  fclose(file);

  print_config(tm);
  fprintf(stdout, "[NeonView] Finished reading the %s config file.\n", tm->tenv->config_file);
}

void write_default_config (TM *tm) {
  FILE *file;
  struct stat st;
  int rv;

  if (stat(tm->tenv->config_dir, &st) == 0) {
    // directory exists
    fprintf(stdout, "[NeonView] Directory %s found.\n", tm->tenv->config_dir);
  } else {
    // directory doesn't exist, create it
    rv = mkdir(tm->tenv->config_dir, 0777);
    if (rv) {
      fprintf(stdout, "[NeonView] WARNING: Could not create directory %s. Check permissions and disk space.\n", tm->tenv->config_dir);
      fprintf(stdout, "[NeonView] WARNING: NeonView will start using default values, but it will probably not be able to save any configuration changes you make.\n");
      return;
    } else {
      fprintf(stdout, "[NeonView] Directory %s created.\n", tm->tenv->config_dir);
    }
  }

  file = fopen(tm->tenv->config_file, "wt");
  if (!file) {
    fprintf(stdout, "[NeonView] WARNING: Could not write default config file. Check permissions and disk space.\n");
    fprintf(stdout, "[NeonView] WARNING: NeonView will start using default values, but it will probably not be able to save any configuration changes you make.\n");
    return;
  }
  // write default config
  fprintf(file, "#########################################################################\n");
  fprintf(file, "# NeonView config file\n");
  fprintf(file, "# Changes done in the program are reflected here.\n");
  fprintf(file, "# You can modify these values, but make sure to read README.txt for instructions\n");
  fprintf(file, "# Everything after # is a comment, double-quotes are not necessary\n");
  fprintf(file, "# NeonView checks this file at start-up, if something is wrong it will load default values\n");
  fprintf(file, "#########################################################################\n");
  fprintf(file, "last_dir=\".\"\n");
  fprintf(file, "start_dir=\".\"\n");
  fprintf(file, "window_width=\"840\"\n");
  fprintf(file, "window_height=\"480\"\n");
  fprintf(file, "start_maximized=\"no\"\n");
  fprintf(file, "show_toolbar=\"yes\"\n");
  fprintf(file, "show_status=\"yes\"\n");
  fprintf(file, "show_left_sidebar=\"yes\"\n");
  fprintf(file, "show_right_sidebar=\"yes\"\n");
  fprintf(file, "remember_fit=\"no\"\n");
  fprintf(file, "fit_next=\"yes\"\n");
  fprintf(file, "remember_zoom=\"no\"\n");
  fprintf(file, "remember_window_size=\"yes\"\n");
  fprintf(file, "window_title=\"NeonView\"\n");
  fprintf(file, "gtk_win_pos=\"GTK_WIN_POS_CENTER\"\n");

  fclose(file);

  fprintf(stdout, "[NeonView] Default config file %s written.\n", tm->tenv->config_file);
}

void load_default_config (TM *tm) {
  init_keys(tm);

  print_config(tm);
  fprintf(stdout, "[NeonView] Default config values loaded.\n");
}

void write_config (TM *tm) {
  FILE *file;
  struct stat st;
  int rv;

  if (stat(tm->tenv->config_dir, &st) == 0) {
    // directory exists
  } else {
    // directory doesn't exist, create it
    rv = mkdir(tm->tenv->config_dir, 0777);
    if (rv) {
      fprintf(stdout, "[NeonView] WARNING: Could not create directory %s. Check permissions and disk space.\n", tm->tenv->config_dir);
      fprintf(stdout, "[NeonView] WARNING: Could not save the settings to %s config file.\n", tm->tenv->config_file);
      return;
    } else {
      // directory created
    }
  }

  file = fopen(tm->tenv->config_file, "wt");
  if (!file) {
    fprintf(stdout, "[NeonView] WARNING: Could not write default config file. Check permissions and disk space.\n");
    fprintf(stdout, "[NeonView] WARNING: Could not save the settings to %s config file.\n", tm->tenv->config_file);
    return;
  }
  // write config file
  char window_width[2560 * 2 + 1];
  char window_height[1440 * 2 + 1];
  sprintf(window_width, "%d", tm->tcfg->window_width);
  sprintf(window_height, "%d", tm->tcfg->window_height);
  fprintf(file, "#########################################################################\n");
  fprintf(file, "# NeonView config file\n");
  fprintf(file, "# Changes done in the program are reflected here.\n");
  fprintf(file, "# You can modify these values, but make sure to read README.txt for instructions\n");
  fprintf(file, "# Everything after # is a comment, double-quotes are not necessary\n");
  fprintf(file, "# NeonView checks this file at start-up, if something is wrong it will load default values\n");
  fprintf(file, "#########################################################################\n");
  fprintf(file, "last_dir=\"%s\"\n", tm->tcfg->last_dir);
  fprintf(file, "start_dir=\"%s\"\n", tm->tcfg->start_dir);
  fprintf(file, "window_width=\"%s\"\n", window_width);
  fprintf(file, "window_height=\"%s\"\n", window_height);
  fprintf(file, "start_maximized=\"%s\"\n", tm->tcfg->start_maximized);
  fprintf(file, "show_toolbar=\"%s\"\n", tm->tcfg->show_toolbar);
  fprintf(file, "show_status=\"%s\"\n", tm->tcfg->show_status);
  fprintf(file, "show_left_sidebar=\"%s\"\n", tm->tcfg->show_left_sidebar);
  fprintf(file, "show_right_sidebar=\"%s\"\n", tm->tcfg->show_right_sidebar);
  fprintf(file, "remember_fit=\"%s\"\n", tm->tcfg->remember_fit);
  fprintf(file, "fit_next=\"%s\"\n", tm->tcfg->fit_next);
  fprintf(file, "remember_zoom=\"%s\"\n", tm->tcfg->remember_zoom);
  fprintf(file, "remember_window_size=\"%s\"\n", tm->tcfg->remember_window_size);
  fprintf(file, "window_title=\"%s\"\n", tm->tcfg->window_title);
  fprintf(file, "gtk_win_pos=\"%s\"\n", tm->tcfg->gtk_win_pos);

  fclose(file);

  fprintf(stdout, "[NeonView] Saved settings to %s config file.\n", tm->tenv->config_file);
}

void get_config (TM *tm) {
  gtk_window_get_size(GTK_WINDOW(tm->tgtk->window), &tm->tcfg->window_width, &tm->tcfg->window_height);
}

void gtk_neon_quit (TM *tm) {
  get_config(tm);
  write_config(tm);

  gtk_main_quit();
}
